'use strict';

var _index = require('../index');

describe('encode', function () {
  it('main-net', function () {
    expect((0, _index.encode)({
      network: '0x1',
      address: '0x00521965e7bd230323c423d96c657db5b79d099f'
    })).toEqual('2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqX');
  });

  it('with genesis hash', function () {
    expect((0, _index.encode)({
      network: '0x94365e3a',
      address: '0x00521965e7bd230323c423d96c657db5b79d099f'
    })).toEqual('5A8bRWU3F7j3REx3vkJWxdjQPp4tqmxFPmab1Tr');
  });

  it('ropsten', function () {
    expect((0, _index.encode)({
      network: '0x3',
      address: '0x00521965e7bd230323c423d96c657db5b79d099f'
    })).toEqual('2oDZvNUgn77w2BKTkd9qKpMeUo8EL94QL5V');
  });

  it('kovan', function () {
    expect((0, _index.encode)({
      network: '0x2a',
      address: '0x00521965e7bd230323c423d96c657db5b79d099f'
    })).toEqual('34ukSmiK1oA1C5Du8aWpkjFGALoH7nsHeDX');
  });

  it('infuranet', function () {
    expect((0, _index.encode)({
      network: '0x16b2',
      address: '0x00521965e7bd230323c423d96c657db5b79d099f'
    })).toEqual('9Xy8yQpdeCNSPGQ9jwTha9MRSb2QJ8HYzf1u');
  });
});

describe('decode', function () {
  it('main-net', function () {
    expect((0, _index.decode)('2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqX')).toEqual({
      network: '0x1',
      address: '0x00521965e7bd230323c423d96c657db5b79d099f'
    });
  });

  it('bad checksum', function () {
    expect(function () {
      (0, _index.decode)('2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqU');
    }).toThrow('Invalid address checksum');
  });

  it('with genesis hash', function () {
    expect((0, _index.decode)('5A8bRWU3F7j3REx3vkJWxdjQPp4tqmxFPmab1Tr')).toEqual({
      network: '0x94365e3a',
      address: '0x00521965e7bd230323c423d96c657db5b79d099f'
    });
  });

  it('ropsten', function () {
    expect((0, _index.decode)('2oDZvNUgn77w2BKTkd9qKpMeUo8EL94QL5V')).toEqual({
      network: '0x3',
      address: '0x00521965e7bd230323c423d96c657db5b79d099f'
    });
  });

  it('kovan', function () {
    expect((0, _index.decode)('34ukSmiK1oA1C5Du8aWpkjFGALoH7nsHeDX')).toEqual({
      network: '0x2a',
      address: '0x00521965e7bd230323c423d96c657db5b79d099f'
    });
  });
});

describe('isMNID', function () {
  it('is valid', function () {
    expect((0, _index.isMNID)('2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqX')).toBeTruthy();
    expect((0, _index.isMNID)('5A8bRWU3F7j3REx3vkJWxdjQPp4tqmxFPmab1Tr')).toBeTruthy();
    expect((0, _index.isMNID)('2oDZvNUgn77w2BKTkd9qKpMeUo8EL94QL5V')).toBeTruthy();
    expect((0, _index.isMNID)('34ukSmiK1oA1C5Du8aWpkjFGALoH7nsHeDX')).toBeTruthy();
    // bad checksum but still MNID
    expect((0, _index.isMNID)('2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqU')).toBeTruthy();
  });

  it('is invalid', function () {
    // Ethereum Hex
    expect((0, _index.isMNID)('0x00521965e7bd230323c423d96c657db5b79d099f')).toBeFalsy();
    // Bitcoin
    expect((0, _index.isMNID)('1GbVUSW5WJmRCpaCJ4hanUny77oDaWW4to')).toBeFalsy();

    // IPFS
    expect((0, _index.isMNID)('QmXuNqXmrkxs4WhTDC2GCnXEep4LUD87bu97LQMn1rkxmQ')).toBeFalsy();

    // Cut off
    expect((0, _index.isMNID)('2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkq')).toBeFalsy();

    expect((0, _index.isMNID)('')).toBeFalsy();
    expect((0, _index.isMNID)(null)).toBeFalsy();
  });
});